// Copyright AudioKit. All Rights Reserved. Revision History at http://github.com/AudioKit/AudioKit/
// This file was auto-autogenerated by scripts and templates at http://github.com/AudioKit/AudioKitDevTools/

import AVFoundation

/// AudioKit version of Apple's HighShelfFilter Audio Unit
///
open class HighShelfFilter: Node, Toggleable {

    fileprivate let effectAU = AVAudioUnitEffect(
    audioComponentDescription:
    AudioComponentDescription(appleEffect: kAudioUnitSubType_HighShelfFilter))

    /// Cut Off Frequency (Hz) ranges from 10000 to 22050 (Default: 10000)
    @Parameter public var cutOffFrequency: AUValue

    /// Gain (dB) ranges from -40 to 40 (Default: 0)
    @Parameter public var gain: AUValue

    /// Tells whether the node is processing (ie. started, playing, or active)
    public var isStarted = true

    /// Initialize the high shelf filter node
    ///
    /// - parameter input: Input node to process
    /// - parameter cutOffFrequency: Cut Off Frequency (Hz) ranges from 10000 to 22050 (Default: 10000)
    /// - parameter gain: Gain (dB) ranges from -40 to 40 (Default: 0)
    ///
    public init(
        _ input: Node,
        cutOffFrequency: AUValue = 10000,
        gain: AUValue = 0) {
        super.init(avAudioNode: effectAU)
        connections.append(input)

        self.$cutOffFrequency.associate(with: effectAU, index: 0)
        self.$gain.associate(with: effectAU, index: 1)

        self.cutOffFrequency = cutOffFrequency
        self.gain = gain
    }

    /// Function to start, play, or activate the node, all do the same thing
    public func start() {
        effectAU.bypass = false
        isStarted = true
    }

    /// Function to stop or bypass the node, both are equivalent
    public func stop() {
        effectAU.bypass = true
        isStarted = false
    }
}
